/* eslint-disable @typescript-eslint/no-explicit-any */
import { View, Text, Platform, TextInput, Keyboard, TouchableOpacity } from 'react-native';
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import {
  BottomSheetModal,
  BottomSheetView,
  BottomSheetBackdrop,
  BottomSheetModalProvider,
} from '@gorhom/bottom-sheet';
import { BottomSheetBackdropProps } from '@gorhom/bottom-sheet/lib/typescript/components/bottomSheetBackdrop/types';
import AnimatedLottieView from 'lottie-react-native';
import { styles } from './styles';
import {
  CodeField,
  Cursor,
  useBlurOnFulfill,
  useClearByFocusCell,
} from 'react-native-confirmation-code-field';
import {
  widthPercentageToDP as wp,
  heightPercentageToDP as hp,
} from 'react-native-responsive-screen';
import PrimaryButton from '@/components/PrimaryButton';
import { mailSort } from '@/utils/mailSort';
import { ErrorFlash } from '@/utils/flashMessage';
import { Constants } from '@/utils/constants';
import Clock from '@/assets/icons/Clock.svg';
import AuthButton from '@/components/AuthButton';

const CELL_COUNT = 5;

interface renderCellProps {
  index: number;
  symbol: string;
  isFocused: boolean;
}

interface PropsTypes {
  bottomSheetModalRef: React.RefObject<BottomSheetModal>;
  email: string;
  onPress?: (value?: string) => void;
  onPressResend?: () => void;
}
const OTPVerificationDialog = (props: PropsTypes) => {
  const { bottomSheetModalRef, email, onPress, onPressResend } = props;
  const [isKeyboardVisible, setKeyboardVisible] = useState(false);

  const buttonRef = useRef<any>();
  // variables
  const snapPoints = useMemo(() => {
    if (isKeyboardVisible) {
      return [Platform.OS == 'android' ? hp(60) : hp(85)]; // Adjust the value as needed when the keyboard is visible
    } else {
      return [hp(55)]; // Default value when the keyboard is hidden
    }
  }, [isKeyboardVisible]);

  const handleSheetChanges = useCallback((index: number) => {
    console.log('handleSheetChanges', index);
    setSeconds(5 * 60);
  }, []);

  const renderBackdrop = useCallback(
    (props: BottomSheetBackdropProps) => (
      <BottomSheetBackdrop
        style={styles.bottomSheetBackdrop}
        {...props}
        disappearsOnIndex={-1}
        appearsOnIndex={0}
        pressBehavior={'close'}
        opacity={0.5}
      />
    ),
    [],
  );

  const [enableMask, setEnableMask] = useState(true);
  const [value, setValue] = useState('');
  const codeFieldRef = useBlurOnFulfill({ value, cellCount: CELL_COUNT });
  const [codeFieldProps, getCellOnLayoutHandler] = useClearByFocusCell({
    value,
    setValue,
  });

  const toggleMask = () => setEnableMask((f) => !f);
  const renderCell = ({ index, symbol, isFocused }: renderCellProps) => {
    let textChild = null;

    if (symbol) {
      textChild = enableMask ? '*' : symbol;
    } else if (isFocused) {
      textChild = <Cursor />;
    }

    return (
      <View
        style={[styles.cell, isFocused && styles.focusCell]}
        onLayout={getCellOnLayoutHandler(index)}
        key={index}
      >
        <Text style={styles.cellText}>{textChild}</Text>
      </View>
    );
  };

  useEffect(() => {
    const keyboardDidShowListener = Keyboard.addListener('keyboardDidShow', () => {
      setKeyboardVisible(true); // Keyboard is visible
    });
    const keyboardDidHideListener = Keyboard.addListener('keyboardDidHide', () => {
      setKeyboardVisible(false); // Keyboard is hidden
    });

    // Cleanup the event listeners when the component is unmounted
    return () => {
      keyboardDidHideListener.remove();
      keyboardDidShowListener.remove();
    };
  }, []);

  const [seconds, setSeconds] = useState(5 * 60); // 5 minutes in seconds
  const [disableResend, setDisableResend] = useState(true);

  useEffect(() => {
    if (seconds === 0) {
      setDisableResend(false);
      return;
    } else {
      const countdown = setInterval(() => {
        setSeconds((prevSeconds) => {
          if (prevSeconds === 0) {
            clearInterval(countdown);
            // Do something when countdown reaches zero
          }
          return prevSeconds - 1;
        });
      }, 1000); // update every second

      return () => clearInterval(countdown);
    }
  }, [seconds]);

  // Convert seconds to minutes and seconds
  const minutes = Math.floor(seconds / 60);
  const remainingSeconds = seconds % 60;

  const onPressResendOtp = () => {
    setSeconds(5 * 60);
    onPressResend && onPressResend();
  };

  const onPressSubmit = () => {
    if (value.split('').length == CELL_COUNT) {
      onPress && onPress(value);
    } else {
      ErrorFlash(Constants.OTP_REQ);
      buttonRef.current['shake']();
    }
  };

  return (
    <BottomSheetModalProvider>
      <BottomSheetModal
        ref={bottomSheetModalRef}
        index={0}
        snapPoints={snapPoints}
        onChange={handleSheetChanges}
        backdropComponent={renderBackdrop}
        keyboardBehavior='fillParent'
        keyboardBlurBehavior='restore'
        backgroundStyle={styles.bottomSheetContainer}
      >
        <BottomSheetView style={styles.bottomSheetView}>
          <Text>We've sent a code to {mailSort(email)}</Text>

          <AnimatedLottieView
            style={styles.animation}
            source={require('@/assets/animation/Otp.json')}
            autoPlay
            speed={0.9}
            loop={true}
          />

          <CodeField
            ref={codeFieldRef}
            {...codeFieldProps}
            value={value}
            onChangeText={setValue}
            cellCount={CELL_COUNT}
            keyboardType='number-pad'
            textContentType='oneTimeCode'
            //  autoComplete={Platform.select({ android: 'sms-otp', default: 'one-time-code' })}
            renderCell={renderCell}
          />

          <View style={styles.resendCodeView}>
            <TouchableOpacity
              style={disableResend && { opacity: 0.5 }}
              disabled={disableResend}
              onPress={onPressResendOtp}
            >
              <Text style={styles.resendCodeText}>Resend Code</Text>
            </TouchableOpacity>
            {seconds == 0 ? (
              <View style={styles.clockView}>
                <Clock />
              </View>
            ) : (
              <Text style={styles.timeText}>
                {' '}
                {`${minutes.toString().padStart(2, '0')}:${remainingSeconds
                  .toString()
                  .padStart(2, '0')}`}
              </Text>
            )}
          </View>

          <AuthButton
            buttonRef={buttonRef}
            style={styles.submitButton}
            title='Submit'
            onPress={onPressSubmit}
          />

          <TouchableOpacity
            style={styles.backButton}
            onPress={() => bottomSheetModalRef.current?.close()}
          >
            <Text style={styles.backButtonText}>Back</Text>
          </TouchableOpacity>
        </BottomSheetView>
      </BottomSheetModal>
    </BottomSheetModalProvider>
  );
};

export default OTPVerificationDialog;
